"""
A simple analog of `GeanyObject` in the C API, that is, an object to emit
all signals on.  The signals are emitted from the C code in signalmanager.c,
where the Geany types get wrapped in PyObject types.
"""

import gobject


class SignalManager(gobject.GObject):
	"""
	Manages callback functions for events emitted by Geany's internal GObject.
	"""
	__gsignals__ = {
		'build-start':				(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										()),
		'document-activate':		(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_PYOBJECT,)),
		'document-before-save':		(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_PYOBJECT,)),
		'document-close':			(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_PYOBJECT,)),
		'document-filetype-set':	(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT)),
		'document-new':				(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_PYOBJECT,)),
		'document-open':			(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_PYOBJECT,)),
		'document-reload':			(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_PYOBJECT,)),
		'document-save':			(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_PYOBJECT,)),
		'editor-notify':			(gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN,
										(gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT)),
		'geany-startup-complete':	(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										()),
		'project-close': 			(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										()),
		'project-dialog-confirmed':	(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_PYOBJECT,)),
		'project-dialog-open':		(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_PYOBJECT,)),
		'project-dialog-close':		(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_PYOBJECT,)),
		'project-open':				(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_PYOBJECT,)),
		'project-save':				(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_PYOBJECT,)),
		'update-editor-menu':		(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
										(gobject.TYPE_STRING, gobject.TYPE_INT,
										gobject.TYPE_PYOBJECT)),
	} # __gsignals__

	def __init__(self):
		self.__gobject_init__()

gobject.type_register(SignalManager)




import geany


class DemoPlugin(geany.Plugin):

    __plugin_name__ = "Demo Plugin"
    __plugin_version__ = "0.01"
    __plugin_description__ = "A Sample plugin"
    __plugin_author__ = "Matthew Brush <mbrush@codebrainz.ca>"

    def __init__(self):
        geany.Plugin.__init__(self)
        print("Demo plugin initializing")
        doc = geany.document.new_file()
        doc.editor.scintilla.set_text("Hello from the Demo plugin")

    def cleanup(self):
        print("Demo plugin cleaning up")



# -*- coding: utf-8 -*-

from geany import glog
from traceback import format_exception
import logging
import sys


GLIB_LOG_LEVEL_MAP = {
	logging.DEBUG: glog.LOG_LEVEL_DEBUG,
	logging.INFO: glog.LOG_LEVEL_INFO,
	logging.WARNING: glog.LOG_LEVEL_WARNING,
	# error and critical levels are swapped on purpose because
	# GLib interprets CRITICAL less fatal than Python and additionally
	# GLib abort()s the program on G_LOG_LEVEL_ERROR which is uncommon
	# in Python
	logging.ERROR: glog.LOG_LEVEL_CRITICAL,
	logging.CRITICAL: glog.LOG_LEVEL_ERROR,
}


class PluginLogger(object):
	"""
	Convenience wrapper softly emulating Python's logging interface.
	Any log messages are passed to the GLib log system using g_log()
	and the LOG_DOMAIN is set automatically to the plugin's name for convenience.
	"""

	def __init__(self, plugin_name):
		self._log_domain = u'geanypy-%s' % plugin_name

	def debug(self, msg_format, *args, **kwargs):
		self.log(logging.DEBUG, msg_format, *args, **kwargs)

	def info(self, msg_format, *args, **kwargs):
		self.log(logging.INFO, msg_format, *args, **kwargs)

	def message(self, msg_format, *args, **kwargs):
		self.log(glog.LOG_LEVEL_MESSAGE, msg_format, *args, **kwargs)

	def warning(self, msg_format, *args, **kwargs):
		self.log(logging.WARNING, msg_format, *args, **kwargs)

	def error(self, msg_format, *args, **kwargs):
		self.log(logging.ERROR, msg_format, *args, **kwargs)

	def exception(self, msg_format, *args, **kwargs):
		kwargs['exc_info'] = True
		self.error(msg_format, *args, **kwargs)

	def critical(self, msg_format, *args, **kwargs):
		self.log(logging.CRITICAL, msg_format, *args, **kwargs)

	warn = warning
	fatal = critical

	def log(self, level, msg_format, *args, **kwargs):
		# map log level from Python to GLib
		glib_log_level = GLIB_LOG_LEVEL_MAP.get(level, glog.LOG_LEVEL_MESSAGE)
		# format message
		log_message = msg_format % args
		# log exception information if requested
		exc_info = kwargs.get('exc_info', False)
		if exc_info:
			traceback_text = self._format_exception(exc_info)
			log_message = '%s\n%s' % (log_message, traceback_text)

		glog.glog(self._log_domain, glib_log_level, log_message)

	def _format_exception(self, exc_info):
		if not isinstance(exc_info, tuple):
			exc_info = sys.exc_info()
		exc_text_lines = format_exception(*exc_info)
		return ''.join(exc_text_lines)



import os
from ConfigParser import SafeConfigParser
import geany
import geany.console
import gobject
import gtk
import gtk.gdk as gdk
import pango


WIDGET_STATES = [ gtk.STATE_NORMAL, gtk.STATE_ACTIVE, gtk.STATE_PRELIGHT,
	gtk.STATE_SELECTED, gtk.STATE_INSENSITIVE ]


class ConsolePlugin(geany.Plugin):

	__plugin_name__ = "Python Console"
	__plugin_description__ = ("Installs a Python console that allows you " +
		"to use the Geany API.")
	__plugin_version__ = "0.01"
	__plugin_author__ = "Matthew Brush <mbrush@codebrainz.ca>"

	_font = "Monospace 9"
	_bg = "#FFFFFF"
	_fg = "#000000"
	_banner = ("Geany Python Console\n You can access the Geany Python " +
				"API by importing the `geany' module.\n")
	_use_rl_completer = True

	_builder = None

	def __init__(self):
		geany.Plugin.__init__(self)
		self.load_config()
		self.install_console()


	def cleanup(self):
		#self.save_config()
		self.on_save_config_timeout() # do it now
		self.uninstall_console()


	def load_config(self):
		self.cfg_path = os.path.join(geany.app.configdir, "plugins", "pyconsole.conf")
		self.cfg = SafeConfigParser()
		self.cfg.read(self.cfg_path)


	def save_config(self):
		gobject.idle_add(self.on_save_config_timeout)


	def on_save_config_timeout(self, data=None):
		self.cfg.write(open(self.cfg_path, 'w'))
		return False


	def install_console(self):

		# load general settings
		self.banner = self.banner
		self.use_rl_completer = self.use_rl_completer

		self.swin = gtk.ScrolledWindow()
		self.swin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
		self.console = geany.console.Console(banner = self.banner,
							use_rlcompleter = self.use_rl_completer)
		self.console.connect("populate-popup", self.on_console_populate_popup)

		# apply appearance settings
		self.font = self.font
		self.bg = self.bg
		self.fg = self.fg

		self.swin.add(self.console)
		geany.main_widgets.message_window_notebook.append_page(self.swin,
			gtk.Label("Python"))

		self.swin.show_all()
		self.save_config()

	def uninstall_console(self):
		self.swin.destroy()


	def _get_font(self):
		if self.cfg.has_section('appearances'):
			if self.cfg.has_option('appearances', 'font'):
				return self.cfg.get('appearances', 'font')
		return self._font
	def _set_font(self, font):
		self._font = font
		font_desc = pango.FontDescription(font)
		self.console.modify_font(font_desc)
		if not self.cfg.has_section('appearances'):
			self.cfg.add_section('appearances')
		self.cfg.set('appearances', 'font', self._font)
		self.save_config()
	font = property(_get_font, _set_font)


	def _get_bg(self):
		if self.cfg.has_section('appearances'):
			if self.cfg.has_option('appearances', 'bg_color'):
				return self.cfg.get('appearances', 'bg_color')
		return self._bg
	def _set_bg(self, bg):
		self._bg = bg
		color = gdk.color_parse(self._bg)
		for state in WIDGET_STATES:
			self.console.modify_bg(state, color)
			self.console.modify_base(state, color)
		if not self.cfg.has_section('appearances'):
			self.cfg.add_section('appearances')
		self.cfg.set('appearances', 'bg_color', self._bg)
		self.save_config()
	bg = property(_get_bg, _set_bg)


	def _get_fg(self):
		if self.cfg.has_section('appearances'):
			if self.cfg.has_option('appearances', 'fg_color'):
				return self.cfg.get('appearances', 'fg_color')
		return self._fg
	def _set_fg(self, fg):
		self._fg = fg
		color = gdk.color_parse(self._fg)
		for state in WIDGET_STATES:
			self.console.modify_fg(state, color)
			self.console.modify_text(state, color)
		if not self.cfg.has_section('appearances'):
			self.cfg.add_section('appearances')
		self.cfg.set('appearances', 'fg_color', self._fg)
		self.save_config()
	fg = property(_get_fg, _set_fg)


	def _get_banner(self):
		if self.cfg.has_section('general'):
				if self.cfg.has_option('general', 'banner'):
					return self.cfg.get('general', 'banner')
		return self._banner
	def _set_banner(self, banner):
		self._banner = banner
		if not self.cfg.has_section('general'):
			self.cfg.add_section('general')
		self.cfg.set('general', 'banner', self._banner)
		self.save_config()
	banner = property(_get_banner, _set_banner)


	def _get_use_rl_completer(self):
		if self.cfg.has_section('general'):
			if self.cfg.has_option('general', 'use_rl_completer'):
				return self.cfg.getboolean('general', 'use_rl_completer')
		return self._use_rl_completer
	def _set_use_rl_completer(self, use_rl_completer):
		self._use_rl_completer = use_rl_completer
		if not self.cfg.has_section('general'):
			self.cfg.add_section('general')
		self.cfg.set('general', 'use_rl_completer',
			str(self._use_rl_completer).lower())
		self.save_config()
	use_rl_completer = property(_get_use_rl_completer, _set_use_rl_completer)


	def on_console_populate_popup(self, textview, menu, data=None):
		item = gtk.SeparatorMenuItem()
		item.show()
		menu.append(item)
		item = gtk.ImageMenuItem(stock_id=gtk.STOCK_PREFERENCES)
		item.show()
		menu.append(item)
		item.connect("activate", lambda w,d=None: self.show_configure())


	def on_banner_changed(self, text_buf, data=None):
		self.banner = text_buf.get_text(text_buf.get_start_iter(), text_buf.get_end_iter())

	def on_use_rl_completer_toggled(self, chk_btn, data=None):
		self.use_rl_completer = chk_btn.get_active()

	def on_font_changed(self, font_btn, data=None):
		self.font = font_btn.get_font_name()

	def on_fg_color_changed(self, clr_btn, data=None):
		self.fg = clr_btn.get_color().to_string()

	def on_bg_color_changed(self, clr_btn, data=None):
		self.bg = clr_btn.get_color().to_string()


	def configure(self, dialog):

		vbox = gtk.VBox(spacing=6)
		vbox.set_border_width(6)

		lbl = gtk.Label()
		lbl.set_use_markup(True)
		lbl.set_markup("<b>General</b>")

		fra_general = gtk.Frame("")
		fra_general.set_shadow_type(gtk.SHADOW_NONE)
		fra_general.set_label_widget(lbl)

		al_general = gtk.Alignment(0.0, 0.0, 1.0, 1.0)
		al_general.set_padding(0, 0, 12, 0)
		fra_general.add(al_general)

		tbl = gtk.Table(3, 2, False)
		tbl.set_row_spacings(6)
		tbl.set_col_spacings(6)
		tbl.set_border_width(6)

		lbl = gtk.Label("Banner:")
		lbl.set_alignment(0.0, 0.0)

		tvw = gtk.TextView()
		tvw.get_buffer().set_text(self.banner)
		tvw.get_buffer().connect("changed", self.on_banner_changed)

		swin = gtk.ScrolledWindow()
		swin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		swin.set_shadow_type(gtk.SHADOW_ETCHED_IN)
		swin.add(tvw)

		tbl.attach(lbl, 0, 1, 0, 1, gtk.FILL, gtk.FILL, 0, 0)
		tbl.attach(swin, 1, 2, 0, 1, gtk.EXPAND | gtk.FILL, gtk.EXPAND | gtk.FILL, 0, 0)

		lbl = gtk.Label("")
		lbl.set_alignment(0.0, 0.5)

		check = gtk.CheckButton("Use Readline")
		if self.use_rl_completer:
			check.set_active(True)
		check.connect("toggled", self.on_use_rl_completer_toggled)

		tbl.attach(lbl, 0, 1, 1, 2, gtk.FILL, gtk.FILL, 0, 0)
		tbl.attach(check, 1, 2, 1, 2, gtk.FILL, gtk.FILL, 0, 0)

		lbl = gtk.Label("")
		lbl.set_alignment(0.0, 0.5)
		lbl.set_use_markup(True)
		lbl.set_markup('<span size="small" style="italic">' +
			'Note: General settings will be applied when console is reloaded.' +
			'</span>')
		tbl.attach(lbl, 0, 2, 2, 3, gtk.FILL, gtk.FILL, 0, 0)

		al_general.add(tbl)

		lbl = gtk.Label()
		lbl.set_use_markup(True)
		lbl.set_markup("<b>Appearances</b>")

		fra_appearances = gtk.Frame("")
		fra_appearances.set_shadow_type(gtk.SHADOW_NONE)
		fra_appearances.set_label_widget(lbl)

		al_appearances = gtk.Alignment(0.0, 0.0, 1.0, 1.0)
		al_appearances.set_padding(0, 0, 12, 0)
		fra_appearances.add(al_appearances)

		tbl = gtk.Table(3, 2, False)
		tbl.set_row_spacings(6)
		tbl.set_col_spacings(6)
		tbl.set_border_width(6)

		lbl = gtk.Label("Font:")
		lbl.set_alignment(0.0, 0.5)

		btn = gtk.FontButton(self.font)
		btn.connect("font-set", self.on_font_changed)

		tbl.attach(lbl, 0, 1, 0, 1, gtk.FILL, gtk.FILL, 0, 0)
		tbl.attach(btn, 1, 2, 0, 1, gtk.FILL | gtk.EXPAND, gtk.FILL, 0, 0)

		lbl = gtk.Label("FG Color:")
		lbl.set_alignment(0.0, 0.5)

		btn = gtk.ColorButton(gdk.color_parse(self.fg))
		btn.connect("color-set", self.on_fg_color_changed)

		tbl.attach(lbl, 0, 1, 1, 2, gtk.FILL, gtk.FILL, 0, 0)
		tbl.attach(btn, 1, 2, 1, 2, gtk.FILL | gtk.EXPAND, gtk.FILL, 0, 0)

		lbl = gtk.Label("BG Color:")
		lbl.set_alignment(0.0, 0.5)

		btn = gtk.ColorButton(gdk.color_parse(self.bg))
		btn.connect("color-set", self.on_bg_color_changed)

		tbl.attach(lbl, 0, 1, 2, 3, gtk.FILL, gtk.FILL, 0, 0)
		tbl.attach(btn, 1, 2, 2, 3, gtk.FILL | gtk.EXPAND, gtk.FILL, 0, 0)

		al_appearances.add(tbl)

		vbox.pack_start(fra_general, True, True, 0)
		vbox.pack_start(fra_appearances, False, True, 0)

		return vbox



"""
All plugins need to import this module and inherit from the Plugin class.  A
basic plugin could look something like this:

	import geany
	from geany.plugin import Plugin

	class OpenNewDocumentPlugin(Plugin):

		__plugin_name__ = "Open new document plugin"

		_doc = None

		def __init__(self):
			Plugin.__init__(self)
			print "About to open a new document"
			self.open_document('/path/to/some/file')

		def open_document(self, filename):
			self._doc = geany.document.open(filename)

		def close_document(self):
			self._doc.close()
			self._doc = None

		def cleanup(self):
			if self._doc is not None:
				self._doc.close()

The guts of the API are exposed to plugins through the `geany` package and
its modules.

Plugins should be placed in either the system plugin directory (something like
/usr/local/lib/geany) or in the user plugin directory (something like
~/.config/geany/plugins).  Only files with a `.py` extension will be loaded.
"""


from geany.logger import PluginLogger
import keybindings

class Plugin(object):
	"""
	Base class for all plugins.  All plugins must inherit from this in order
	to be properly detected.
	"""

	# Child classes should implement these (at least __plugin__name__)
	#__plugin_name__ = None
	#__plugin_description__ = None
	#__plugin_version__ = None
	#__plugin_author__ = None

	_events = {
		"document-open": [],
		# TODO: add more events here
	}


	def __init__(self):
		"""
		When the plugin is loaded its __init__() function will be called
		so that's a good place to put plugin initialization code.
		"""
		self.logger = PluginLogger(self.name)


	def cleanup(self):
		"""
		When the plugin is unloaded the cleanup() function will be called, so
		it's a good place to put and clean-up/tear-down code.
		"""
		pass


	@property
	def __plugin_name__(self):
		"""
		Plugins must implement a __plugin_name__ attribute that returns the
		string name of the plugin.
		"""
		raise NotImplementedError(
			"Plugins must implement the __plugin_name__ attribute.")


	@property
	def name(self):
		"""
		Get plugin's name.
		"""
		return self.__plugin_name__


	@property
	def description(self):
		"""
		Get plugin's description.
		"""
		if hasattr(self, '__plugin_description__'):
			return self.__plugin_description__
		else:
			return ""


	@property
	def version(self):
		"""
		Get plugin's version.
		"""
		if hasattr(self, '__plugin_version__'):
			return self.__plugin_version__
		else:
			return ""


	@property
	def author(self):
		"""
		Get plugin's author.
		"""
		if hasattr(self, '__plugin_author__'):
			return self.__plugin_author__
		else:
			return ""

	def set_key_group(self, section_name, count, callback = None):
		"""
		Sets up a GeanyKeyGroup for this plugin. You can use that group to add keybindings
		with group.add_key_item().
		"""
		return keybindings.set_key_group(self, section_name, count, callback)



import gtk
import geany

class HelloWorld(geany.Plugin):

    __plugin_name__ = "HelloWorld"
    __plugin_version__ = "1.0"
    __plugin_description__ = "Just another tool to say hello world"
    __plugin_author__ = "John Doe <john.doe@example.org>"

    def __init__(self):
        self.menu_item = gtk.MenuItem("Hello World")
        self.menu_item.show()
        geany.main_widgets.tools_menu.append(self.menu_item)
        self.menu_item.connect("activate", self.on_hello_item_clicked)

    def cleanup(self):
        self.menu_item.destroy()

    def on_hello_item_clicked(widget, data):
        geany.dialogs.show_msgbox("Hello World")



"""
Package file that exposes some of Geany's guts as its own attibutes.  Any
objects where it only makes sense to have one instance of are initialed here
and set as attributes.

You can sort of think of this file as the GeanyData struct from the C API,
though no real attempt is made to mimic that structure here.
"""

import app
import console
import dialogs
import document
import editor
import encoding
import filetypes
import highlighting
import glog
import main
import msgwindow
import navqueue
import prefs
import project
import scintilla
import search
import templates
import ui_utils
import keybindings

from app import App
from prefs import Prefs, ToolPrefs
from main import is_realized, locale_init, reload_configuration
from signalmanager import SignalManager
from ui_utils import MainWidgets, InterfacePrefs
from search import SearchPrefs
from templates import TemplatePrefs


__all__ = [ "Plugin",
            "is_realized",
            "locale_init",
            "reload_configuration",
            "main_widgets",
            "interface_prefs",
            "app",
            "glog",
            "keybindings",
            "general_prefs",
            "search_prefs",
            "template_prefs",
            "tool_prefs",
            "signals" ]

# Geany's application data fields
app = App()

# Import GTK+ widgets that are part of Geany's UI
main_widgets = MainWidgets()

# Preferences
general_prefs = Prefs() # GeanyData->prefs but name clashes with module
interface_prefs = InterfacePrefs()
search_prefs = SearchPrefs()
template_prefs = TemplatePrefs()
tool_prefs = ToolPrefs()

# GObject to connect signal handlers on and which emits signals.
signals = SignalManager()

import plugin
from plugin import Plugin
